//---------------------------------------------------------------------------
// Copyright (C) 2000 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  initrov.c - This utility uses TMEX to initialize the roving SHA iButton 
//              (DS1963S).
//
//  Compiler:  Microsoft Visual C 
//  Version:   1.02
//  History:   
//             1.01 -> 1.02  Used correct MUC (Monetary Units Code) and 
//                           multiplier for US dollars
//

#define TMEXUTIL

#include "ibtmexcw.h"   
#include "ibsha.h"

// external function prototypes 
extern int RunScriptSHA(Script script[], void *, int);
extern int PrintHeader(char *);
extern void output_status(int, char *);
extern int EnterString(char *, char *, int, int);
extern int ToHex(char ch);
extern int GetSeviceProviderSettings(int, char *, uchar *, int, uchar *);
extern int GetRovingSettings(uchar *, ulong *);
extern int WriteFileSHA(uchar *, uchar *, int, uchar *);
extern int bitacc(int, int, int, uchar *);
extern void ExitProg(char *, int);

// local function
int CreateRovingSHA(uchar *, uchar *, ulong, char *);

//----------------------------------------------------------------------
//  This is the Main routine for initrov 
//
void main(short argc, char **argv)
{
   int ShowStatus=FALSE,filenum;
   char msg[300];
   uchar auth_secret[85],dmmy[85];
   int cnt,exit_code,i;
   ulong money;
   uchar rov_information[128];

   // check arguments to see if request instruction with '?' or too many
   if ((argc > 3) || ((argc > 1) && (argv[1][0] == '?' || argv[1][1] == '?')))
       ExitProg("\nusage: initrov <output_filename> </Verbose>\n"
              "  - initialize a roving SHA iButton on default\n"
              "    1-Wire Net port (use SetPrt32.exe to set)\n"
              "    co-processor SHA iButton must be present\n"
              "  - <output_filename> optional output filename\n"
              "  - </V> optional show verbose output (default not shown)\n"
              "  - version 1.02\n",EXIT_ERROR);

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum[IB_ROVING],&PortType[IB_ROVING]))
      ExitProg("ERROR, Could not read the default PortNum and PortType!\n",EXIT_ERROR);

   // hard code CO_PROCESSOR port to be the same
   PortNum[IB_COPR] = PortNum[IB_ROVING];
   PortType[IB_COPR] = PortType[IB_ROVING];

   // print the header info 
   if (!PrintHeader("InitRov Version 1.02"))
      ExitProg("No driver found\n",EXIT_ERROR);

   // check arguments 
   VERBOSE = FALSE;
   filenum = 0;
   if (argc >= 2)
   {
      if (argv[1][0] != '/')
         filenum = 1;
      else if ((argv[1][1] == 'V') || (argv[1][1] == 'v'))
         VERBOSE = TRUE;

      if (argc == 3)
      {    
         if (argv[2][0] != '/')
            filenum = 2;
         else if ((argv[2][1] == 'V') || (argv[2][1] == 'v'))
            VERBOSE = TRUE;
      }   
   }

   // open the output file  
   fp = NULL;
   if (filenum > 0)
   {
      fp = fopen(argv[filenum],"wb");
      if(fp == NULL)
      {    
         printf("ERROR, Could not open output file!\n");
         exit(EXIT_ERROR);
      }
      else
         printf("File '%s' opened to write sensor data.\n\n",
                 argv[filenum]);
   }

   // run the script to setup the 1-Wire
   if (RunScriptSHA(Verify1Wire, NULL, 0))
      output_status(LV_ALWAYS,"** 1-Wire port verified\n\n");
   else
      ExitProg("1-Wire Net not successfuly setup, end program\n",EXIT_ERROR);

   // attempt to open session on 1-Wire net
   if (RunScriptSHA(OpenSession, NULL, 0))
   {
      output_status(LV_VERBOSE,"** Session on 1-Wire opened\n");

      // run the script find the co-processor
      provider[0] = 0;
      if (RunScriptSHA(FindCoprocessor, &copr_rom[0], sizeof(copr_rom)))
      {
         // see if provider has been set (real result of FindCopr)
         if (provider[0])
         {
            // loop to print ROM of device found
            cnt = sprintf(msg,"** Co-processor SHA iButton found: ");
            for (i = 7; i >= 0; i--)
               cnt += sprintf(msg + cnt,"%02X",copr_rom[i]);
            cnt += sprintf(msg + cnt,"\n** Provider file: ");
            for (i = 0; i < 4; i++)
               cnt += sprintf(msg + cnt,"%c",provider[i]);
            cnt += sprintf(msg + cnt,".%d\n",provider[4]);
            cnt += sprintf(msg + cnt,"** Money master secret page: %d\n",c_mmaster_scrt);
            cnt += sprintf(msg + cnt,"** Auth master secret page: %d\n",c_amaster_scrt);
            cnt += sprintf(msg + cnt,"** Unique device (calculated) secret page: %d\n",c_udevice_scrt);
            output_status(LV_ALWAYS,msg);
            exit_code = EXIT_SUCCESS;
         }
         else
         {
            output_status(LV_ALWAYS,"ERROR, could not find co-processor on 1-Wire Net, end program\n");
            exit_code = EXIT_ERROR;
         }
      }
      else
      {
         output_status(LV_ALWAYS,"ERROR, could not find co-processor on 1-Wire Net, end program\n");
         exit_code = EXIT_ERROR;
      }

      // close session
      if (RunScriptSHA(CloseSession, NULL, 0))
         output_status(LV_VERBOSE,"** Session on 1-Wire closed\n");
   }
   else
   {
      output_status(LV_ALWAYS,"ERROR, Could not open co-processor 1-Wire port\n");
      exit_code = EXIT_ERROR;
   }

   // check on result of operation
   if (exit_code)
   {
      printf("ERROR reported (%d), end program\n",exit_code);
      // end the program
      exit(exit_code);
   }

   // prompt for service provider settings
   output_status(LV_ALWAYS,"\n\nEnter provider information to create the roving SHA iButton\n");
   if (!GetSeviceProviderSettings(FALSE, provider, auth_secret, FALSE, dmmy))
      ExitProg("Aborted entering the provider settings, end program\n",EXIT_ERROR);

   // destination roving page (fixed pages for demo)
   r_udevice_scrt = 5;  // roving unique device secret

   // default info and money
   sprintf(rov_information,"no info");  
   money = 1000;  

   // loop to create roving SHA iButtons
   do
   {
      // create roving SHA
      exit_code = CreateRovingSHA(&auth_secret[0], &rov_information[0], money, &msg[0]);
      output_status(LV_ALWAYS,msg);

      // print clear result
      if (exit_code == EXIT_SUCCESS)
         output_status(LV_ALWAYS,"\n-------------\n   SUCCESS\n-------------\n");
      else if (exit_code != EXIT_ERROR)
         output_status(LV_ALWAYS,"\n-------------\n    ERROR\n-------------\n");
   }
   while (!kbhit() && (exit_code != EXIT_ERROR));

   // close opened file
   if ((fp != NULL) && (fp != stdout))
   {
      printf("File '%s' closed.\n",
              argv[filenum]);
      fclose(fp);
   }

   // check on result of operation
   if (exit_code)
      printf("ERROR reported (%d), end program\n",exit_code);
   else
      printf("End program normally\n");

   // end the program
   exit(exit_code);
}


//--------------------------------------------------------------------------
// Find and create a roving sha iButtons
//
int CreateRovingSHA(uchar *auth_secret, uchar *rov_information, ulong money, char *msg)
{
   int rt=EXIT_SUCCESS,cnt=0,i;
   ulong ul;
   short ROM[8];
   uchar page_map[5];
   eCertificate ec;

   // place in loop for easy cleanup on error
   for (;;)
   {
      // prompt for the roving information
      if (!GetRovingSettings(rov_information, &money))
      {
         cnt += sprintf(msg + cnt,"Aborted entering the roving settings, end program\n");
         rt = EXIT_ERROR;
         break;
      }

      // give warning
      output_status(LV_ALWAYS,"\n** Warning (non-Copr) SHA iButton on 1-Wire will be overwritten!!\n"
                    "   (Press any key)\n\n");
      MessageBeep(MB_OK);  MessageBeep(MB_OK);
      getch();
   
      // attempt to open session on 1-Wire net
      if (RunScriptSHA(OpenSession, NULL, 0))
         cnt += sprintf(msg + cnt,"\n\n** Session on 1-Wire opened\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, Could not open co-processor 1-Wire port\n");
         rt = EXIT_ERROR;
         break;
      }
      
      // select roving port
      if (RunScriptSHA(SelectRoving, NULL, 0))
         cnt += sprintf(msg + cnt,"** 1-Wire port for roving selected\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, Could not select co-processor 1-Wire\n");
         rt = EXIT_ERROR;
         break;
      }

      // find the first available (non-Copr) SHA iButton
      if (RunScriptSHA(FindSHA, &ROM[0], sizeof(ROM)))
      {
         // loop to print ROM of device found
         cnt += sprintf(msg + cnt,"** SHA iButton found: ");
         for (i = 7; i >= 0; i--)
            cnt += sprintf(msg + cnt,"%02X",ROM[i]);
         cnt += sprintf(msg + cnt,"\n");

         // set page map for money file 
         page_map[0] = r_udevice_scrt;
         page_map[1] = r_udevice_scrt + 8;
      }
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, Could not find a SHA iButton\n");
         rt = EXIT_ERROR_NO_SHA;
         break;
      }

      // create the authentication secret
      if (RunScriptSHA(CreateUniqueDeviceSecret, &auth_secret[0], 32))
         cnt += sprintf(msg + cnt,"** Uniqued device secret created\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, Could not create authorization secret\n");
         rt = EXIT_ERROR_AUTH_SCRT;
         break;
      }

      // write the money/info file (with invalid money to start)
      if (WriteFileSHA(provider, rov_information, 56, page_map))
         cnt += sprintf(msg + cnt,"** Roving info file created\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, Could not write roving info file\n");
         rt = EXIT_ERROR_WRITE;
         break;
      }
                  
      // read auth money page (to get counters/rom in ec) 
      if (RunScriptSHA(ReadAuth, &ec, sizeof(ec)))
      {
         cnt += sprintf(msg + cnt,"** Money page read to get counter\n");
         // put correct money value in cert
         ul = money * 100;  // (1.02)
         for (i = 0; i < 3; i++)
         {
            ec.balance[i] = (uchar)(ul & 0xFF);
            ul >>= 8;
         }
         // increment counter 
         ec.write_cycle_cnt++;
         // set transaction ID
         ec.trans_id = 0x1234;
         // code mult
         ec.code_mult = 0x8B48;  // US Dollars with 10^-2 multiplier (1.02)
      }
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, could not read auth money page\n"
                      "   Auth secret provide may be incorrect!!\n");
         rt = EXIT_ERROR_WRITE;
         break;
      }

      // write real money 
      if (RunScriptSHA(WriteMoney, &ec, sizeof(ec)))
         cnt += sprintf(msg + cnt,"** Money written\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, Could not write money\n");
         rt = EXIT_ERROR_WRITE;
         break;
      }

      // read auth money page (to get copr to do unique secret) 
      if (RunScriptSHA(ReadAuth, &ec, sizeof(ec)))
         cnt += sprintf(msg + cnt,"** Money page authenticate read\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, could not read auth money page\n"
                      "   Auth secret provide may be incorrect!!\n");
         rt = EXIT_ERROR_WRITE;
         break;
      }

      // verify money (sanity check) 
      if (RunScriptSHA(VerifyMoney, &ec, sizeof(ec)))
         cnt += sprintf(msg + cnt,"** Money contents verified\n");
      else
      {
         cnt += sprintf(msg + cnt,"ERROR, could not verify money\n");
         rt = EXIT_ERROR_WRITE;
         break;
      }

      break;
   }
   
   // close session (cleanup)
   if (RunScriptSHA(CloseSession, NULL, 0))
      cnt += sprintf(msg + cnt,"** Session on 1-Wire closed\n");
   else
      cnt += sprintf(msg + cnt,"ERROR, closing 1-Wire Session\n");

   return rt;
}